# -*- coding: utf-8 -*-
"""
apapi.basic_connection
~~~~~~~~~~~~~~~~
This module provides a Basic Connection class,
which should be used to connect to Anaplan APIs
"""

import base64
import threading
import time

from requests import Response, Session

from .authentication import AnaplanAuth
from .utils import AuthType, AUTH_URL, API_URL, get_generic_session


class BasicConnection:
    """Basic Anaplan connection session. Provides authentication and requesting."""

    def __init__(
        self,
        credentials: str,
        auth_type: AuthType = AuthType.BASIC,
        session: Session = get_generic_session(),
        auth_url: str = AUTH_URL,
        api_url: str = API_URL,
    ):

        self._credentials = credentials
        self._auth_type = auth_type
        self._auth_url = auth_url
        self._api_main_url = f"{api_url}/2/0"
        self._timer = None
        self._lock = threading.Lock()

        self.details: bool = True
        self.compress: bool = True
        self.timeout: float = 3.5
        self.session: Session = session

        self.authenticate()

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.close()

    def _handle_token(self, token_info: dict) -> None:
        self.session.auth = AnaplanAuth("AnaplanAuthToken " + token_info["tokenValue"])
        # Anaplan yields "expiresAt" in ms, that's why we need to divide it by 1000
        self._timer = threading.Timer(
            token_info["expiresAt"] / 1000 - time.time(), self.refresh_token
        )
        self._timer.start()

    def authenticate(self) -> None:
        """Acquire Anaplan Authentication Service Token"""
        if self._auth_type == AuthType.BASIC:
            auth_string = str(
                base64.b64encode(self._credentials.encode("utf-8")).decode("utf-8")
            )
            headers = self.session.headers.copy()
            headers["Authorization"] = "Basic " + auth_string
            response = self.session.post(
                f"{self._auth_url}/token/authenticate",
                headers=headers,
                timeout=self.timeout,
            )
            if not response.ok:
                raise Exception("Unable to authenticate", response.text)
        elif self._auth_type == AuthType.CERT:
            raise NotImplementedError(
                "Certificate authentication has not been implemented yet"
            )
        else:
            raise Exception("Raise exception - unsupported auth type/wrong format")
        self._handle_token(response.json()["tokenInfo"])

    def refresh_token(self) -> None:
        """Refresh Anaplan Authentication Service Token"""
        # skip if other thread is already taking care of refreshing the token
        if not self._lock.locked():
            with self._lock:
                response = self.session.post(
                    f"{self._auth_url}/token/refresh", timeout=self.timeout
                )
                if not response.ok:
                    raise Exception("Unable to refresh the token", response.text)
                self._timer.cancel()
                self._handle_token(response.json()["tokenInfo"])

    def close(self) -> None:
        """Logout from Anaplan Authentication Service"""
        self._timer.cancel()
        self.session.post(f"{self._auth_url}/token/logout", timeout=self.timeout)
        self.session.close()

    def request(
        self, method: str, url: str, params: dict = None, data=None, headers=None
    ) -> Response:
        """Default wrapper of session's request method"""
        if headers:
            response = self.session.request(
                method, url, params, data, timeout=self.timeout, headers=headers
            )
        else:
            response = self.session.request(
                method, url, params, data, timeout=self.timeout
            )
        if not response.ok:
            raise Exception("Request failed", url, response.text)
        return response



# -*- coding: utf-8 -*-
"""
apapi.utils
~~~~~~~~~~~
This module provides utility functions that are used within APAPI,
but might also be useful for external consumption.
"""

import enum

from requests import Session
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

from .__version__ import __title__, __version__

AUTH_URL = "https://auth.anaplan.com"
API_URL = "https://api.anaplan.com"

USER_AGENT = f"{__title__}/{__version__}"
APP_JSON = "application/json"
APP_8STREAM = "application/octet-stream"
APP_GZIP = "application/x-gzip"
TEXT_CSV = "text/csv"
TEXT_CSV_ESCAPED = "text/csv;escaped=true"
ENCODING_GZIP = "gzip,deflate"
DEFAULT_HEADERS = {
    "Content-Type": APP_JSON,
    "Accept": APP_JSON,
    "User-Agent": USER_AGENT,
}
DEFAULT_DATA = {"localeName": "en_US"}


class AuthType(enum.Enum):
    """Needed during initialization of connection to choose
    which authentication option should be used"""

    BASIC = 1
    CERT = 2


class ExportType(enum.Enum):
    """Needed for large cell view read to choose which format should be used"""

    GRID = "GRID_ALL_PAGES"
    TABULAR_SINGLE = "TABULAR_SINGLE_COLUMN"
    TABULAR_MULTI = "TABULAR_MULTI_COLUMN"


def get_generic_session(retry_count: int = 3) -> Session:
    """Returns default session: headers & adapter (with given retry count) mounted"""
    adapter = HTTPAdapter(
        max_retries=Retry(
            total=retry_count,
            allowed_methods=None,  # this means retry on ANY method (including POST)
            status_forcelist=(407, 410, 429, 500, 502, 503, 504),
        )
    )
    session = Session()
    session.mount("http://", adapter)
    session.mount("https://", adapter)
    session.headers = DEFAULT_HEADERS.copy()
    return session



# -*- coding: utf-8 -*-
"""
.. include:: ../README.md
"""
from .__version__ import (
    __title__,
    __description__,
    __url__,
    __version__,
    __author__,
    __author_email__,
    __license__,
    __copyright__,
)
from . import utils
from .authentication import AnaplanAuth
from .basic_connection import BasicConnection
from .bulk import BulkConnection
from .transactional import TransactionalConnection
from .connection import Connection

# Set default logging handler to avoid "No handler found" warnings.
import logging

logging.getLogger(__name__).addHandler(logging.NullHandler())



# -*- coding: utf-8 -*-
import argparse
import sys

from apapi import __title__, __description__, __version__


def main():
    parser = argparse.ArgumentParser(
        prog=__title__,
        description=__description__,
        epilog="""Usage should be similar to official Anaplan Connect:
               https://anaplanenablement.s3.amazonaws.com/Community/Anapedia/Anaplan_Connect_User_Guide_v3.0.0.pdf""",
    )
    # General
    parser.add_argument(
        "-version", "--version", action="version", version=f"%(prog)s {__version__}"
    )
    parser.add_argument("-help", action="help", help=argparse.SUPPRESS)
    parser.add_argument(
        "-d", "-debug", action="store_true", help="Flag: Enable debugging"
    )
    parser.add_argument(
        "-q", "-quiet", action="store_true", help="Flag: Reduce verbosity"
    )
    # Connection config
    parser.add_argument(
        "-u",
        "-user",
        required=True,
        help="Auth: Provide basic auth info (password can be also given later)",
        metavar="{username:password}",
    )
    parser.add_argument(
        "-v", "-via", help="Proxy: Use specified proxy settings", metavar="{proxy}"
    )
    parser.add_argument(
        "-vu",
        "-viauser",
        help="Proxy: Use proxy configuration {domain/workstation optional}",
        metavar="{domain/workstation/username:password}",
    )
    parser.add_argument(
        "-auth",
        "-authServiceUrl",
        help="Config: Use specified authentication service",
        metavar="{url}",
    )
    parser.add_argument(
        "-s", "-service", help="Config: Use specified API service", metavar="{url}"
    )
    parser.add_argument(
        "-ct",
        "-httptimeout",
        type=float,
        help="Config: Use connection timeout as specified",
        metavar="{seconds}",
    )
    parser.add_argument(
        "-mrc",
        "-maxretrycount",
        type=int,
        help="Config: Use retry count as specified",
        metavar="{count}",
    )
    parser.add_argument(
        "-rt",
        "-retrytimeout",
        type=float,
        help="Config: Use retry timeout as specified",
        metavar="{seconds}",
    )
    # Chaining IDs
    parser.add_argument(
        "-w", "-workspace", help="Info: Use specified workspace", metavar="{ID/name}"
    )
    parser.add_argument(
        "-m", "-model", help="Info: Use specified model", metavar="{ID/name}"
    )
    parser.add_argument(
        "-l", "-list", help="Info: Use specified list", metavar="{ID/name}"
    )
    parser.add_argument(
        "-mo", "-module", help="Info: Use specified module", metavar="{ID/name}"
    )
    parser.add_argument(
        "-vi", "-view", help="Info: Use specified view", metavar="{ID/name}"
    )
    # Additional arguments & output config
    parser.add_argument(
        "-chunksize",
        type=float,
        help="Info: Use given max chunk size for imports",
        metavar="{MBs}",
    )
    parser.add_argument(
        "-pages",
        help="Info: Use given pages selection for cells read",
        metavar="{pages}",
    )
    parser.add_argument(
        "-xm",
        "-mappingproperty",
        help="Info: Use given selection for import",
        metavar="{dimension:item}",
    )
    parser.add_argument(
        "-xl",
        "-locale",
        help="Info: Use specified ISO language and country code",
        metavar="{lang_country}",
    )
    parser.add_argument(
        "-o",
        "-output",
        help="Info: Save errors dump to specified path",
        metavar="{path}",
    )
    parser.add_argument(
        "-f", "-file", help="Info: Use specified file", metavar="{ID/name}"
    )
    parser.add_argument(
        "-im",
        "-itemmappingproperty",
        help="Info: Path to mapping between local and Anaplan columns",
        metavar="{path}",
    )
    # Actions
    action_group = parser.add_mutually_exclusive_group()
    action_group.add_argument(
        "-i", "-import", help="Action: Run specified import", metavar="{ID/name}"
    )
    action_group.add_argument(
        "-e", "-export", help="Action: Run specified export", metavar="{ID/name}"
    )
    action_group.add_argument(
        "-a", "-action", help="Action: Run specified action", metavar="{ID/name}"
    )
    action_group.add_argument(
        "-pr", "-process", help="Action: Run specified process", metavar="{ID/name}"
    )
    action_group.add_argument(
        "-I",
        "-imports",
        action="store_true",
        help="Action: Get available model imports definitions",
    )
    action_group.add_argument(
        "-E",
        "-exports",
        action="store_true",
        help="Action: Get available model exports definitions",
    )
    action_group.add_argument(
        "-A",
        "-actions",
        action="store_true",
        help="Action: Get available model actions definitions",
    )
    action_group.add_argument(
        "-P",
        "-processes",
        action="store_true",
        help="Action: Get available model processes definitions",
    )
    action_group.add_argument(
        "-F",
        "-files",
        action="store_true",
        help="Action: Get available model files definitions",
    )
    action_group.add_argument(
        "-W",
        "-workspaces",
        action="store_true",
        help="Action: Get available workspaces",
    )
    action_group.add_argument(
        "-M", "-models", action="store_true", help="Action: Get available models"
    )
    action_group.add_argument(
        "-L", "-lists", action="store_true", help="Action: Get available model lists"
    )
    action_group.add_argument(
        "-MO",
        "-modules",
        action="store_true",
        help="Action: Get available model modules",
    )
    action_group.add_argument(
        "-V", "-views", action="store_true", help="Action: Get available model views"
    )
    action_group.add_argument(
        "-putItems:csv", help="Action: Add items to list from CSV", metavar="{path}"
    )
    action_group.add_argument(
        "-putItems:json", help="Action: Add items to list from JSON", metavar="{path}"
    )
    action_group.add_argument(
        "-updateItems:csv", help="Action: Update list items from CSV", metavar="{path}"
    )
    action_group.add_argument(
        "-updateItems:json",
        help="Action: Update list items from JSON",
        metavar="{path}",
    )
    action_group.add_argument(
        "-upsertItems:csv",
        help="Action: Upsert (add or update) list items from CSV",
        metavar="{path}",
    )
    action_group.add_argument(
        "-upsertItems:json",
        help="Action: Upsert (add or update) list items from JSON",
        metavar="{path}",
    )
    action_group.add_argument(
        "-deleteItems:csv",
        help="Action: Delete items from list from CSV",
        metavar="{path}",
    )
    action_group.add_argument(
        "-deleteItems:json",
        help="Action: Delete items from list from JSON",
        metavar="{path}",
    )
    # Input options
    put_group = parser.add_mutually_exclusive_group()
    put_group.add_argument(
        "-p",
        "-put",
        help="Input option: Get data from specified path",
        metavar="{path}",
    )
    put_group.add_argument(
        "-putc",
        action="store_true",
        help="Input option: Get data as tab-separated from terminal",
    )
    put_group.add_argument(
        "-puts", action="store_true", help="Input option: Get data from terminal"
    )
    # Output options
    get_group = parser.add_mutually_exclusive_group()
    get_group.add_argument(
        "-t", "-get", help="Output option: write to specified path", metavar="{path}"
    )
    get_group.add_argument(
        "-getc",
        action="store_true",
        help="Output option: Display file content as tab-separated in terminal",
    )
    get_group.add_argument(
        "-gets",
        action="store_true",
        help="Output option: Display file content in terminal",
    )
    get_group.add_argument(
        "-get:csv",
        help="Output option: write to specified path as CSV",
        metavar="{path}",
    )
    get_group.add_argument(
        "-get:csv_sc",
        help="Output option: write single-column view to specified path as CSV",
        metavar="{path}",
    )
    get_group.add_argument(
        "-get:csv_mc",
        help="Output option: write multi-column view to specified path as CSV",
        metavar="{path}",
    )
    get_group.add_argument(
        "-get:json",
        help="Output option: write to specified path as JSON",
        metavar="{path}",
    )
    # Execute types
    execute_group = parser.add_mutually_exclusive_group()
    execute_group.add_argument(
        "-x", "-execute", action="store_true", help="Execute type: execute action"
    )
    execute_group.add_argument(
        "-x:all",
        "-execute:all",
        action="store_true",
        help="Execute type: get list items with properties",
    )
    execute_group.add_argument(
        "-emd", action="store_true", help="Execute type: get export's details"
    )

    args_dict = vars(parser.parse_args(args=None if sys.argv[1:] else ["-h"]))

    raise NotImplementedError("Apapi module execution is not supported yet.")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
apapi.transactional
~~~~~~~~~~~~~~~~
Child of Basic Connection class, responsible for Transactional API capabilities
"""

import json

from requests import Response

from .basic_connection import BasicConnection
from .utils import APP_JSON, TEXT_CSV, ExportType, ENCODING_GZIP


class TransactionalConnection(BasicConnection):
    """Anaplan connection with Transactional API functions."""

    # Users
    def get_users(self) -> Response:
        return self.request("GET", f"{self._api_main_url}/users")

    def get_me(self) -> Response:
        return self.request("GET", f"{self._api_main_url}/users/me")

    def get_user(self, user_id: str) -> Response:
        return self.request("GET", f"{self._api_main_url}/users/{user_id}")

    def get_workspace_users(self, workspace_id: str) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/workspaces/{workspace_id}/users",
        )

    def get_workspace_admins(self, workspace_id: str) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/workspaces/{workspace_id}/admins",
            {"limit": 2147483647},  # 2^31-1, max accepted, put here as default is 20
        )

    def get_model_users(self, model_id: str) -> Response:
        return self.request("GET", f"{self._api_main_url}/models/{model_id}/users")

    # Workspaces
    def get_workspaces(self, details: bool = None) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/workspaces",
            {"tenantDetails": self.details if details is None else details},
        )

    def get_workspace(self, workspace_id: str, details: bool = None) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/workspaces/{workspace_id}",
            {"tenantDetails": self.details if details is None else details},
        )

    # Models
    def get_models(self, details: bool = None) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models",
            {"modelDetails": self.details if details is None else details},
        )

    def get_workspace_models(self, workspace_id: str, details: bool = None) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/workspaces/{workspace_id}/models",
            {"modelDetails": self.details if details is None else details},
        )

    def get_model(self, model_id: str, details: bool = None) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}",
            {"modelDetails": self.details if details is None else details},
        )

    # Calendar
    def get_fiscal_year(self, model_id: str):
        return self.request(
            "GET", f"{self._api_main_url}/models/{model_id}/modelCalendar"
        )

    def set_fiscal_year(self, model_id: str, data: str):
        return self.request(
            "PUT",
            f"{self._api_main_url}/models/{model_id}/modelCalendar/fiscalYear",
            data=json.dumps({"year": data}),
        )

    def get_current_period(self, model_id: str):
        return self.request(
            "GET", f"{self._api_main_url}/models/{model_id}/currentPeriod"
        )

    def set_current_period(self, model_id: str, data: str):
        return self.request(
            "PUT",
            f"{self._api_main_url}/models/{model_id}/currentPeriod",
            data=json.dumps({"date": data}),
        )

    # Versions
    def get_versions(self, model_id: str):
        return self.request("GET", f"{self._api_main_url}/models/{model_id}/versions")

    def set_version_switchover(self, model_id: str, version_id: str, data: str):
        return self.request(
            "PUT",
            f"{self._api_main_url}/models/{model_id}/versions/{version_id}/switchover",
            data=json.dumps({"date": data}),
        )

    # Lists
    def get_lists(self, model_id: str) -> Response:
        return self.request("GET", f"{self._api_main_url}/models/{model_id}/lists")

    def get_list(self, model_id: str, list_id: str) -> Response:
        return self.request(
            "GET", f"{self._api_main_url}/models/{model_id}/lists/{list_id}"
        )

    def get_list_items(
        self, model_id: str, list_id: str, details: bool = None, accept: str = None
    ) -> Response:
        if accept:
            headers = self.session.headers.copy()
            headers["Accept"] = accept
        else:
            headers = None
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/lists/{list_id}/items",
            {"includeAll": self.details if details is None else details},
            headers=headers,
        )

    def start_large_list_read(self, model_id: str, list_id: str) -> Response:
        return self.request(
            "POST",
            f"{self._api_main_url}/models/{model_id}/lists/{list_id}/readRequests",
        )

    def get_large_list_read_status(
        self, model_id: str, list_id: str, request_id: str
    ) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/lists/{list_id}/readRequests/{request_id}",
        )

    def get_large_list_read_data(
        self,
        model_id: str,
        list_id: str,
        request_id: str,
        page: str,
        compress: bool = None,
    ) -> Response:
        headers = self.session.headers.copy()
        headers["Accept"] = TEXT_CSV
        if compress or (compress is None and self.compress):
            headers["Accept-Encoding"] = ENCODING_GZIP
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/lists/{list_id}/readRequests/{request_id}/pages/{page}",
            headers=headers,
        )

    def delete_large_list_read(
        self, model_id: str, list_id: str, request_id: str
    ) -> Response:
        return self.request(
            "DELETE",
            f"{self._api_main_url}/models/{model_id}/lists/{list_id}/readRequests/{request_id}",
        )

    def add_list_items(self, model_id: str, list_id: str, data: list[dict]) -> Response:
        return self.request(
            "POST",
            f"{self._api_main_url}/models/{model_id}/lists/{list_id}/items",
            {"action": "add"},
            json.dumps({"items": data}),
        )

    def update_list_items(
        self, model_id: str, list_id: str, data: list[dict]
    ) -> Response:
        return self.request(
            "PUT",
            f"{self._api_main_url}/models/{model_id}/lists/{list_id}/items",
            data=json.dumps({"items": data}),
        )

    def delete_list_items(
        self, model_id: str, list_id: str, data: list[dict]
    ) -> Response:
        return self.request(
            "POST",
            f"{self._api_main_url}/models/{model_id}/lists/{list_id}/items",
            {"action": "delete"},
            json.dumps({"items": data}),
        )

    def reset_list_index(self, model_id: str, list_id: str) -> Response:
        return self.request(
            "POST", f"{self._api_main_url}/models/{model_id}/lists/{list_id}/resetIndex"
        )

    # Modules
    def get_modules(self, model_id: str) -> Response:
        return self.request("GET", f"{self._api_main_url}/models/{model_id}/modules")

    # Lineitems
    def get_lineitems(self, model_id: str, details: bool = None) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/lineItems",
            {"includeAll": self.details if details is None else details},
        )

    def get_module_lineitems(
        self, model_id: str, module_id: str, details: bool = None
    ) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/modules/{module_id}/lineItems",
            {"includeAll": self.details if details is None else details},
        )

    # Views
    def get_views(self, model_id: str, details: bool = None) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/views",
            {"includesubsidiaryviews": self.details if details is None else details},
        )

    def get_module_views(
        self, model_id: str, module_id: str, details: bool = None
    ) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/modules/{module_id}/views",
            {"includesubsidiaryviews": self.details if details is None else details},
        )

    def get_view_dimensions(self, model_id: str, view_id: str) -> Response:
        return self.request(
            "GET", f"{self._api_main_url}/models/{model_id}/views/{view_id}"
        )

    # Dimensions
    def get_dimension_items(self, model_id: str, dimension_id: str) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/dimensions/{dimension_id}/items",
        )

    def get_lineitem_dimensions(self, model_id: str, lineitem_id: str) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/lineItems/{lineitem_id}/dimensions",
        )

    def get_lineitem_dimension_items(
        self, model_id: str, lineitem_id: str, dimension_id: str
    ) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/lineItems/{lineitem_id}/dimensions/{dimension_id}/items",
        )

    def get_view_dimension_items(
        self, model_id: str, view_id: str, dimension_id: str
    ) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/views/{view_id}/dimensions/{dimension_id}/items",
        )

    def check_dimension_items_id(
        self, model_id: str, dimension_id: str, data: dict
    ) -> Response:
        return self.request(
            "POST",
            f"{self._api_main_url}/models/{model_id}/dimensions/{dimension_id}/items",
            data=json.dumps(data),
        )

    # Cells
    def get_cell_data(
        self, model_id: str, view_id: str, accept: str = None, pages: dict[str] = None
    ) -> Response:
        headers = self.session.headers.copy()
        if accept:
            headers["Accept"] = accept
        params = {}
        if pages:
            params["pages"] = pages
        if headers["Accept"] == APP_JSON:
            params["format"] = "v1"
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/views/{view_id}/data",
            params=params,
            headers=headers,
        )

    def start_large_cell_read(
        self, model_id: str, view_id: str, mode: ExportType
    ) -> Response:
        return self.request(
            "POST",
            f"{self._api_main_url}/models/{model_id}/views/{view_id}/readRequests",
            data=json.dumps({"exportType": mode.value}),
        )

    def get_large_cell_read_status(
        self, model_id: str, view_id: str, request_id: str
    ) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/views/{view_id}/readRequests/{request_id}",
        )

    def get_large_cell_read_data(
        self,
        model_id: str,
        view_id: str,
        request_id: str,
        page: str,
        compress: bool = None,
    ) -> Response:
        headers = self.session.headers.copy()
        headers["Accept"] = TEXT_CSV
        if compress or (compress is None and self.compress):
            headers["Accept-Encoding"] = ENCODING_GZIP
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/views/{view_id}/readRequests/{request_id}/pages/{page}",
            headers=headers,
        )

    def delete_large_cell_read(
        self, model_id: str, view_id: str, request_id: str
    ) -> Response:
        return self.request(
            "DELETE",
            f"{self._api_main_url}/models/{model_id}/views/{view_id}/readRequests/{request_id}",
        )

    def post_cell_data(
        self, model_id: str, module_id: str, data: list[dict]
    ) -> Response:
        return self.request(
            "POST",
            f"{self._api_main_url}/models/{model_id}/modules/{module_id}/data",
            data=json.dumps(data),
        )



# -*- coding: utf-8 -*-
__title__ = "apapi"
__description__ = "Unofficial Anaplan API Python client library"
__url__ = "https://github.com/DLZaan/apapi"
__version__ = "0.2.0"
__author__ = "Milosz Kierepka"
__author_email__ = "milosz.kierepka+work@gmail.com"
__license__ = "Apache 2.0"
__copyright__ = "Copyright 2022 Milosz Kierepka"



# -*- coding: utf-8 -*-
"""
apapi.authentication
~~~~~~~~~~~~~~~~
This module provides an AnaplanAuth object to use for requests authentication
"""

from requests.auth import AuthBase


class AnaplanAuth(AuthBase):
    """Allows to easily use Anaplan token for authentication"""

    def __init__(self, token):
        self.token = token

    def __call__(self, r):
        r.headers["Authorization"] = self.token
        return r



# -*- coding: utf-8 -*-
"""
apapi.bulk
~~~~~~~~~~~~~~~~
Child of Basic Connection class, responsible for Bulk API capabilities
"""
import json

from requests import Response

from .basic_connection import BasicConnection
from .utils import APP_8STREAM, APP_GZIP, DEFAULT_DATA


class BulkConnection(BasicConnection):
    """Anaplan connection with Bulk API functions."""

    # Actions
    def _get_actions(self, model_id: str, action_type: str) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/{action_type}",
        )

    def get_imports(self, model_id: str) -> Response:
        return self._get_actions(model_id, "imports")

    def get_exports(self, model_id: str) -> Response:
        return self._get_actions(model_id, "exports")

    def get_actions(self, model_id: str) -> Response:
        return self._get_actions(model_id, "actions")

    def get_processes(self, model_id: str) -> Response:
        return self._get_actions(model_id, "processes")

    def get_files(self, model_id: str) -> Response:
        return self._get_actions(model_id, "files")

    # Action details
    def get_import(self, model_id: str, import_id: str) -> Response:
        return self.request(
            "GET", f"{self._api_main_url}/models/{model_id}/imports/{import_id}"
        )

    def get_export(self, model_id: str, export_id: str) -> Response:
        return self.request(
            "GET", f"{self._api_main_url}/models/{model_id}/exports/{export_id}"
        )

    def get_process(
        self, model_id: str, process_id: str, details: bool = None
    ) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/processes/{process_id}",
            {"showImportDataSource": self.details if details is None else details},
        )

    # Files manipulation
    def set_file_chunk_count(self, model_id: str, file_id: str, count: int) -> Response:
        return self.request(
            "POST",
            f"{self._api_main_url}/models/{model_id}/files/{file_id}",
            data=json.dumps({"chunkCount": count}),
        )

    def upload_file_chunk(
        self,
        model_id: str,
        file_id: str,
        data: bytes,
        chunk: int,
        compressed: bool = False,
    ) -> Response:
        headers = self.session.headers.copy()
        headers["Content-Type"] = APP_GZIP if compressed else APP_8STREAM
        return self.request(
            "PUT",
            f"{self._api_main_url}/models/{model_id}/files/{file_id}/chunks/{chunk}",
            data=data,
            headers=headers,
        )

    def set_file_upload_complete(self, model_id: str, file_id: str) -> Response:
        return self.request(
            "POST",
            f"{self._api_main_url}/models/{model_id}/files/{file_id}/complete",
            data=json.dumps({"id": file_id}),
        )

    def upload_file(self, model_id: str, file_id: str, data: bytes) -> Response:
        headers = self.session.headers.copy()
        headers["Content-Type"] = APP_8STREAM
        return self.request(
            "PUT",
            f"{self._api_main_url}/models/{model_id}/files/{file_id}",
            data=data,
            headers=headers,
        )

    def get_file(self, model_id: str, file_id: str) -> Response:
        headers = self.session.headers.copy()
        headers["Accept"] = APP_8STREAM
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/files/{file_id}",
            headers=headers,
        )

    def download_file(self, model_id: str, file_id: str) -> bytes:
        url = f"{self._api_main_url}/models/{model_id}/files/{file_id}/chunks"
        response = self.request("GET", url)
        if not response.json()["meta"]["paging"]["currentPageSize"]:
            raise Exception("Missing part in request response", url, response.text)
        headers = self.session.headers.copy()
        headers["Accept"] = APP_8STREAM
        return b"".join(
            self.request("GET", f"{url}/{chunk_id['id']}", headers=headers).content
            for chunk_id in response.json()["chunks"]
        )

    def delete_file(self, model_id: str, file_id: str) -> Response:
        return self.request(
            "DELETE",
            f"{self._api_main_url}/models/{model_id}/files/{file_id}",
        )

    # Run action
    def _run_action(
        self,
        model_id: str,
        action_id: str,
        action_type: str,
        data=None,
    ) -> Response:
        if data is None:
            data = DEFAULT_DATA.copy()
        return self.request(
            "POST",
            f"{self._api_main_url}/models/{model_id}/{action_type}/{action_id}/tasks",
            data=json.dumps(data),
        )

    def run_import(self, model_id: str, import_id: str, data=None) -> Response:
        return self._run_action(model_id, import_id, "imports", data)

    def run_export(self, model_id: str, export_id: str) -> Response:
        return self._run_action(model_id, export_id, "exports")

    def run_action(self, model_id: str, action_id: str) -> Response:
        return self._run_action(model_id, action_id, "actions")

    def run_process(self, model_id: str, process_id: str, data=None) -> Response:
        return self._run_action(model_id, process_id, "processes", data)

    # Get tasks
    def _get_action_tasks(
        self, model_id: str, action_id: str, action_type: str
    ) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/{action_type}/{action_id}/tasks",
        )

    def get_import_tasks(self, model_id: str, import_id: str) -> Response:
        return self._get_action_tasks(model_id, import_id, "imports")

    def get_export_tasks(self, model_id: str, export_id: str) -> Response:
        return self._get_action_tasks(model_id, export_id, "exports")

    def get_action_tasks(self, model_id: str, action_id: str) -> Response:
        return self._get_action_tasks(model_id, action_id, "actions")

    def get_process_tasks(self, model_id: str, process_id: str) -> Response:
        return self._get_action_tasks(model_id, process_id, "processes")

    # Get task
    def _get_action_task(
        self, model_id: str, action_id: str, task_id: str, action_type: str
    ) -> Response:
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/{action_type}/{action_id}/tasks/{task_id}",
        )

    def get_import_task(self, model_id: str, import_id: str, task_id: str) -> Response:
        return self._get_action_task(model_id, import_id, task_id, "imports")

    def get_export_task(self, model_id: str, export_id: str, task_id: str) -> Response:
        return self._get_action_task(model_id, export_id, task_id, "exports")

    def get_action_task(self, model_id: str, action_id: str, task_id: str) -> Response:
        return self._get_action_task(model_id, action_id, task_id, "actions")

    def get_process_task(
        self, model_id: str, process_id: str, task_id: str
    ) -> Response:
        return self._get_action_task(model_id, process_id, task_id, "processes")

    # Get dump
    def get_import_dump(self, model_id: str, import_id: str, task_id: str) -> Response:
        headers = self.session.headers.copy()
        headers["Accept"] = APP_8STREAM
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/imports/{import_id}/tasks/{task_id}/dump",
            headers=headers,
        )

    def download_import_dump(
        self, model_id: str, import_id: str, task_id: str
    ) -> bytes:
        url = f"{self._api_main_url}/models/{model_id}/imports/{import_id}/tasks/{task_id}/dump/chunks"
        response = self.request("GET", url)
        if not response.json()["meta"]["paging"]["currentPageSize"]:
            raise Exception("Missing part in request response", url, response.text)
        headers = self.session.headers.copy()
        headers["Accept"] = APP_8STREAM
        return b"".join(
            self.request("GET", f"{url}/{chunk_id['id']}", headers=headers).content
            for chunk_id in response.json()["chunks"]
        )

    def get_process_dump(
        self, model_id: str, process_id: str, task_id: str, object_id: str
    ) -> Response:
        headers = self.session.headers.copy()
        headers["Accept"] = APP_8STREAM
        return self.request(
            "GET",
            f"{self._api_main_url}/models/{model_id}/processes/{process_id}/tasks/{task_id}/dumps/{object_id}",
            headers=headers,
        )

    def download_process_dump(
        self, model_id: str, process_id: str, task_id: str, object_id: str
    ) -> bytes:
        url = f"{self._api_main_url}/models/{model_id}/processes/{process_id}/tasks/{task_id}/dumps/{object_id}/chunks"
        response = self.request("GET", url)
        if not response.json()["meta"]["paging"]["currentPageSize"]:
            raise Exception("Missing part in request response", url, response.text)
        headers = self.session.headers.copy()
        headers["Accept"] = APP_8STREAM
        return b"".join(
            self.request("GET", f"{url}/{chunk_id['id']}", headers=headers).content
            for chunk_id in response.json()["chunks"]
        )



# -*- coding: utf-8 -*-
"""
apapi.connection
~~~~~~~~~~~~~~~~
This module provides Connection class, which contains all available API functions
"""

from .bulk import BulkConnection
from .transactional import TransactionalConnection


class Connection(
    BulkConnection,
    TransactionalConnection,
):
    """Anaplan connection session. Provides all available API functions."""



# -*- coding: utf-8 -*-
import os
from setuptools import find_packages, setup

REQUIRES_PYTHON = ">=3.9.0"
REQUIRES = ["requests>=2.27"]

here = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(here, "apapi", "__version__.py"), "r", encoding="utf-8") as f:
    exec(f.read(), about)

try:
    with open(os.path.join(here, "README.md"), "r", encoding="utf-8") as f:
        long_description = "\n" + f.read()
except FileNotFoundError:
    long_description = about["__description__"]

setup(
    name=about["__title__"],
    version=about["__version__"],
    description=about["__description__"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=about["__author__"],
    author_email=about["__author_email__"],
    maintainer=about["__author__"],
    maintainer_email=about["__author_email__"],
    url=about["__url__"],
    license=about["__license__"],
    platforms="any",
    keywords="anaplan anaplanapi anaplanconnector client",
    packages=find_packages(),
    include_package_data=True,
    python_requires=REQUIRES_PYTHON,
    install_requires=REQUIRES,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Office/Business :: Financial",
        "Topic :: Software Development :: Libraries",
    ],
    entry_points={
        "console_scripts": [
            "apapi=apapi.__main__:main",
        ]
    },
)
